package com.qizhen.healsphere.repository.neo4j;

import cn.hutool.core.collection.CollUtil;
import com.qizhen.healsphere.config.Neo4jUtil;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.repository.neo4j.param.MergeNodeParam;
import com.qizhen.healsphere.web.dto.NodeDTO;
import org.neo4j.driver.v1.Record;
import org.neo4j.driver.v1.Session;
import org.neo4j.driver.v1.StatementResult;
import org.neo4j.driver.v1.Values;
import org.neo4j.driver.v1.types.Node;
import org.neo4j.driver.v1.types.Relationship;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class BaseNodeRepository {

    @Autowired
    private Neo4jUtil neo4jUtil;

    /**
     * 根据名字和标签查询实体属性
     * @param label
     * @param name
     * @return 实体map
     */
    public Map<String,Object> findNodeByName(String label,String name) {
        String query = String.format("MATCH (n:`%s`) WHERE n.name = $name and (n.is_deleted = 'N' or NOT exists(n.is_deleted)) RETURN n", label);
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("name", name));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;

        }

    }

    public List<NodeDTO> findNodeByNameLike(String label,String name) {
        String query = String.format("MATCH (n:`%s`) WHERE n.name contains $name and (n.is_deleted = 'N' or NOT exists(n.is_deleted)) return distinct n.name as name,id(n) as id,head(Labels(n)) as label limit 20", label);
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("name", name));
            List<NodeDTO> nodeDTOS = new ArrayList<>();
            if(result.hasNext()) {
                List<Record> records = result.list();
                for (Record record : records) {
                    NodeDTO nodeDTO = new NodeDTO();
                    nodeDTO.setName(record.get("name").asString());
                    nodeDTO.setNodeId(record.get("id").asLong());
                    nodeDTO.setLabel(record.get("label").asString());
                    nodeDTOS.add(nodeDTO);
                }
            }
            return nodeDTOS;
        }

    }

    /**
     * 根据id查询实体属性
     * @param id
     * @return 实体map
     */
    public Map<String,Object> findNodeById(Long id) {
        String query = "MATCH (n) WHERE id(n) = $nodeId and (n.is_deleted = 'N' or NOT exists(n.is_deleted)) RETURN n";
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("nodeId", id));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;

        }

    }

    /**
     * 根据名字模糊查询实体
     * @param label
     * @param name
     * @return 实体map列表
     */
    public List<Map<String,Object>> findNodeListByName(String label, String name) {
        String query = String.format("MATCH (n:`%s`) WHERE n.name contains $name and (n.is_deleted = 'N' or NOT exists(n.is_deleted)) RETURN n", label);

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("name", name));
            List<Map<String,Object>> listMap = new ArrayList<>();
            if(result.hasNext()) {
                List<Record> records = result.list();
                for (Record record : records) {
                    Map<String,Object> map = new HashMap<>();
                    Node node = record.get("n").asNode();
                    map.put("id", node.id());
                    map.put("labels", node.labels());
                    map.put("properties", node.asMap());
                    listMap.add(map);
                }
            }
            return listMap;

        }
    }

    /**
     * 新增实体
     * @param label
     * @param properties
     * @return 实体map
     */
    public Map<String, Object> createNode(String label, Map<String, Object> properties) {
        StringBuilder createClause = new StringBuilder();
        if (CollUtil.isEmpty(properties)) {
            properties = new HashMap<>();
        }
        properties.put("is_deleted","N");
        properties.forEach((key, value) -> createClause.append(key).append(": '").append(value).append("'").append(", "));

        // 移除最后的逗号
        if (createClause.length() > 0) {
            createClause.setLength(createClause.length() - 2);
        }

        String query = String.format("MERGE (n:`%s` { %s }) RETURN n", label, createClause);

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query);
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;
        }
    }

    /**
     * 更新实体名称
     * @param nodeId
     * @param name
     * @return 实体map
     */
    public Map<String, Object> updateNode(Long nodeId, String name) {
        String query = "MATCH (n) WHERE id(n) = $nodeId SET n.name=$name RETURN n";
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("nodeId", nodeId,"name", name));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;
        }
    }

    /**
     * 删除实体
     * @param nodeId
     */
    public void deleteNode(Long nodeId) {
        String query = "MATCH (n) WHERE id(n) = $nodeId SET n.is_deleted = 'Y' " +
                "WITH n MATCH (n)-[r]-() " +
                "DELETE r";
        try(Session session = neo4jUtil.getSession()) {
            session.run(query,Values.parameters("nodeId", nodeId));
        }
    }

    /**
     * 实体合并
     */
    public Map<String, Object> mergeNode(MergeNodeParam mergeNodeParam) {
        try(Session session = neo4jUtil.getSession()) {
            // 找到要合并的两个节点
            String firstQuery = String.format("MATCH (a:`%s`) WHERE id(a) = $firstId RETURN a", mergeNodeParam.getFirstLabel());
            String secondQuery = String.format("MATCH (b:`%s`) WHERE id(b) = $secondId RETURN b", mergeNodeParam.getSecondLabel());
            
            StatementResult aResult = session.run(firstQuery, Values.parameters("firstId", mergeNodeParam.getFirstId()));
            StatementResult bResult = session.run(secondQuery, Values.parameters("secondId", mergeNodeParam.getSecondId()));
            
            // 检查节点是否存在
            if (!aResult.hasNext() || !bResult.hasNext()) {
                throw new RuntimeException("One or both nodes not found");
            }

            // 创建新节点
            Map<String, Object> newNodeProps = new HashMap<>();
            newNodeProps.put("name", mergeNodeParam.getNewName());
            StatementResult newNodeResult = session.run(
                String.format("CREATE (n:`%s` {name:$name}) RETURN n", mergeNodeParam.getNewLabel()),
                Values.parameters("name", newNodeProps.get("name"))
            );
            Node newNode = newNodeResult.single().get("n").asNode();
            long newNodeId = newNode.id();

            // 处理第一个节点的关系
            processNodeRelations(session, mergeNodeParam.getFirstLabel(), mergeNodeParam.getFirstId(), newNodeId);
            
            // 处理第二个节点的关系
            processNodeRelations(session, mergeNodeParam.getSecondLabel(), mergeNodeParam.getSecondId(), newNodeId);

            // 删除原始节点
            session.run(
                String.format("MATCH (a:`%s`) WHERE id(a) = $firstId DETACH DELETE a", mergeNodeParam.getFirstLabel()),
                Values.parameters("firstId", mergeNodeParam.getFirstId())
            );
            session.run(
                String.format("MATCH (a:`%s`) WHERE id(a) = $secondId DETACH DELETE a", mergeNodeParam.getSecondLabel()),
                Values.parameters("secondId", mergeNodeParam.getSecondId())
            );

            Map<String,Object> map = new HashMap<>();
            map.put("id", newNode.id());
            map.put("labels", newNode.labels());
            map.put("properties", newNode.asMap());
            return map;
        }
    }

    /**
     * 处理节点的所有关系
     */
    private void processNodeRelations(Session session, String label, Long sourceNodeId, Long newNodeId) {
        // 获取所有关系类型
        String getRelationsQuery = String.format(
            "MATCH (a:`%s`)-[r]-() WHERE id(a) = $nodeId " +
            "WITH DISTINCT type(r) as relType " +
            "ORDER BY relType " +
            "RETURN relType", 
            label
        );

        // 使用cursor处理关系类型
        StatementResult relTypes = session.run(getRelationsQuery, 
            Values.parameters("nodeId", sourceNodeId));
        
        while (relTypes.hasNext()) {
            String relType = relTypes.next().get("relType").asString();
            
            // 使用cursor处理出向关系
            String outRelQuery = String.format(
                "MATCH (a:`%s`)-[r:`%s`]->(other) WHERE id(a) = $sourceId " +
                "WITH r, other " +
                "ORDER BY id(other) " +
                "RETURN r, other",
                label, relType
            );
            
            StatementResult outRels = session.run(outRelQuery, 
                Values.parameters("sourceId", sourceNodeId));
            
            while (outRels.hasNext()) {
                Record record = outRels.next();
                Relationship rel = record.get("r").asRelationship();
                Node other = record.get("other").asNode();
                
                // 创建新关系
                String createRelQuery = String.format(
                    "MATCH (n), (other) WHERE id(n) = $newNodeId AND id(other) = $otherId " +
                    "MERGE (n)-[newRel:`%s`]->(other) " +
                    "SET newRel += $props",
                    relType
                );
                
                session.run(createRelQuery, Values.parameters(
                    "newNodeId", newNodeId,
                    "otherId", other.id(),
                    "props", rel.asMap()
                ));
            }
            
            // 使用cursor处理入向关系
            String inRelQuery = String.format(
                "MATCH (a:`%s`)<-[r:`%s`]-(other) WHERE id(a) = $sourceId " +
                "WITH r, other " +
                "ORDER BY id(other) " +
                "RETURN r, other",
                label, relType
            );
            
            StatementResult inRels = session.run(inRelQuery, 
                Values.parameters("sourceId", sourceNodeId));
            
            while (inRels.hasNext()) {
                Record record = inRels.next();
                Relationship rel = record.get("r").asRelationship();
                Node other = record.get("other").asNode();
                
                // 创建新关系
                String createRelQuery = String.format(
                    "MATCH (n), (other) WHERE id(n) = $newNodeId AND id(other) = $otherId " +
                    "MERGE (other)-[newRel:`%s`]->(n) " +
                    "SET newRel += $props",
                    relType
                );
                
                session.run(createRelQuery, Values.parameters(
                    "newNodeId", newNodeId,
                    "otherId", other.id(),
                    "props", rel.asMap()
                ));
            }
        }
    }

    /**
     * 新增实体
     * @param label
     * @param properties
     * @return 实体map
     */
    public BaseEntity create(String label, Map<String, Object> properties) {
        StringBuilder createClause = new StringBuilder();
        properties.put("is_deleted","N");
        properties.forEach((key, value) -> createClause.append(key).append(": '").append(value).append("'").append(", "));

        // 移除最后的逗号
        if (createClause.length() > 0) {
            createClause.setLength(createClause.length() - 2);
        }

        String query = String.format("MERGE (n:`%s` { %s }) RETURN n", label, createClause);

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query);

            if(result.hasNext()) {
                return buildBaseEntity(result.single().get("n").asNode());
            }
            return null;
        }
    }

    /**
     * 根据名字和标签查询实体属性
     * @param label
     * @param name
     * @return 实体map
     */
    public BaseEntity byName(String label,String name) {
        String query = String.format("MATCH (n:`%s`) WHERE n.name = $name and (n.is_deleted = 'N' or NOT exists(n.is_deleted)) RETURN n", label);
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("name", name));
            if(result.hasNext()) {
                return buildBaseEntity(result.single().get("n").asNode());
            }

            return null;
        }

    }

    public List<BaseEntity> getAll(String label) {
        String query = String.format("MATCH (n:`%s`) WHERE (n.is_deleted = 'N' or NOT exists(n.is_deleted)) RETURN n", label);
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query);
            if(result.hasNext()) {
                List<Record> records = result.list();
                List<BaseEntity> r = new ArrayList<>();
                for (Record record : records) {
                    r.add(buildBaseEntity(record.get("n").asNode()));
                }
                return r;
            }
            return null;
        }

    }

    public static BaseEntity buildBaseEntity(Node node) {
        BaseEntity baseEntity = new BaseEntity();
        baseEntity.setId(node.id());
        baseEntity.setLabel((ArrayList)node.labels());
        baseEntity.setName(node.get("name").asString());
        Map<String, Object> map = new HashMap<>(node.asMap());
        map.remove("name");
        baseEntity.setProperties(map);
        return baseEntity;
    }

    /**
     * 获取所有科室及其对应的疾病树形结构
     * @return 所有科室-疾病树形结构的列表
     */
    public List<Map<String, Object>> getAllDepartmentDiseaseTrees() {
        String query = 
            "MATCH (d:就诊科室)-[r:疾病相关就诊科室]-(disease:疾病) " +
            "WHERE (d.is_deleted = 'N' or NOT exists(d.is_deleted)) " +
            "AND (disease.is_deleted = 'N' or NOT exists(disease.is_deleted)) " +
            "WITH d.name as department, collect({name: disease.name, sNode: []}) as diseases " +
            "RETURN {name: department, sNode: diseases} as tree";
        
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query);
            
            List<Map<String, Object>> trees = new ArrayList<>();
            while(result.hasNext()) {
                Record record = result.next();
                Map<String, Object> tree = record.get("tree").asMap();
                trees.add(tree);
            }
            return trees;
        }
    }

    public BaseEntity renameLabel(String newLabel,String oldLabel,Long id) {
        String query = String.format("MATCH (n:`%s`) WHERE id(n)=$id SET n:`%s` REMOVE n:`%s` RETURN n", oldLabel,newLabel,oldLabel);
        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("id", id));
            if(result.hasNext()) {
                List<Record> records = result.list();
                for (Record record : records) {
                    return buildBaseEntity(record.get("n").asNode());
                }
            }
            return null;
        }

    }
}