package com.qizhen.healsphere.util;

import cn.hutool.core.util.StrUtil;
import com.qizhen.healsphere.config.Neo4jUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.neo4j.driver.v1.Session;
import org.neo4j.driver.v1.Values;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Component
public class IcdRelationCreator {

    @Autowired
    private Neo4jUtil neo4jUtil;

    private static final int BATCH_SIZE = 1000;

    public void createIcdRelations() {
        log.info("开始创建ICD编码关系...");
        
        try (FileInputStream fis = new FileInputStream("/Users/ycw/IdeaProjects/healsphere/src/main/resources/疾病带编码-20250113.xlsx");
             Workbook workbook = new XSSFWorkbook(fis)) {
            
            Sheet sheet = workbook.getSheetAt(0);  // 获取第一个工作表
            int count = 0;
            
            try (Session session = neo4jUtil.getSession()) {
                // 跳过表头
                boolean isFirstRow = true;
                
                for (Row row : sheet) {
                    if (isFirstRow) {
                        isFirstRow = false;
                        continue;
                    }
                    
                    // 读取单元格数据
                    String diseaseName = getCellValueAsString(row.getCell(0));
                    String icdMedical = getCellValueAsString(row.getCell(1));
                    String icdClinical = getCellValueAsString(row.getCell(2));
                    
                    if (StrUtil.isNotBlank(diseaseName)) {
                        // 只处理有效的ICD编码
                        if (StrUtil.isNotBlank(icdMedical)) {
                            createIcdRelation(session, diseaseName, icdMedical, "医保2.0", "疾病相关ICD10医保2.0编码");
                        }
                        
                        if (StrUtil.isNotBlank(icdClinical)) {
                            createIcdRelation(session, diseaseName, icdClinical, "临床2.0", "疾病相关ICD10临床2.0编码");
                        }
                        
                        count++;
                        if (count % BATCH_SIZE == 0) {
                            log.info("已处理 {} 条记录", count);
                        }
                    }
                }
            }
            
            log.info("ICD编码关系创建完成，共处理 {} 条记录", count);
            
        } catch (IOException e) {
            log.error("创建ICD编码关系时发生错误", e);
            throw new RuntimeException("创建ICD编码关系失败", e);
        }
    }
    
    private String getCellValueAsString(Cell cell) {
        if (cell == null) {
            return "";
        }
        
        switch (cell.getCellType()) {
            case STRING:
                return cell.getStringCellValue().trim();
            case NUMERIC:
                // 处理数字格式的单元格
                double value = cell.getNumericCellValue();
                if (DateUtil.isCellDateFormatted(cell)) {
                    return cell.getLocalDateTimeCellValue().toString();
                }
                // 如果是整数，去掉小数点和0
                if (value == Math.floor(value)) {
                    return String.format("%.0f", value);
                }
                return String.valueOf(value);
            case BOOLEAN:
                return String.valueOf(cell.getBooleanCellValue());
            case FORMULA:
                try {
                    return cell.getStringCellValue();
                } catch (Exception e) {
                    try {
                        return String.valueOf(cell.getNumericCellValue());
                    } catch (Exception ex) {
                        return "";
                    }
                }
            default:
                return "";
        }
    }
    
    private void createIcdRelation(Session session, String diseaseName, String icdCode, String icdType, String relationType) {
        // 为ICD编码创建属性
        Map<String, Object> icdProperties = new HashMap<>();
        icdProperties.put("name", icdCode);
        icdProperties.put("type", icdType);  // 用type属性区分医保编码和临床编码
        
        // Cypher查询语句
        String cypher = 
            "MATCH (d:疾病 {name: $diseaseName}) " +
            "MERGE (i:ICD10编码 {name: $icdCode}) " +  // 使用单一标签
            "SET i += $icdProperties " +
            "MERGE (d)-[r:`" + relationType + "`]->(i) " +
            "RETURN d, i, r";
        
        // 执行Cypher查询
        session.run(cypher, Values.parameters(
            "diseaseName", diseaseName,
            "icdCode", icdCode,
            "icdProperties", icdProperties
        ));
    }
} 